﻿using System;
using System.Collections.Generic;
using System.Linq;
using TSLab.DataSource;

// ReSharper disable UnusedAutoPropertyAccessor.Global
// ReSharper disable MemberCanBePrivate.Global
// ReSharper disable UnusedMember.Global
namespace TSLab.Script.Handlers
{
    [HandlerDecimals(0)]
    [HandlerCategory("TradeMath")]
    public class Volume : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Volume);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Time : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return
                new List<double>(from b in source.Bars
                                 select (b.Date.Hour*10000.0 + b.Date.Minute*100.0 + b.Date.Second));
        }
    }

    [HandlerCategory("TradeMath")]
    public class Date : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return
                new List<double>(from b in source.Bars
                                 select ((b.Date.Year % 100) * 10000.0 + b.Date.Month * 100.0 + b.Date.Day));
        }
    }

    [HandlerName("Day Of Week")]
    [HandlerCategory("TradeMath")]
    public class DayOfWeek : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return
                new List<double>(
                    source.Bars.Select(b => ((b.Date.DayOfWeek == System.DayOfWeek.Sunday
                                                  ? 7.0
                                                  : (double) b.Date.DayOfWeek))));
        }
    }

    [HandlerCategory("TradeMath")]
    public class Open : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Open);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Close : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Close);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Low : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Low);
        }
    }

    [HandlerCategory("TradeMath")]
    public class High : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.High);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Bid : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Bid);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Ask : IBar2DoubleHandler
    {
        public IList<double> Execute(ISecurity source)
        {
            return new List<double>(from b in source.Bars select b.Ask);
        }
    }

    [HandlerName("Constant")]
    [HandlerCategory("TradeMath")]
    public class ConstGen : IBar2DoubleHandler, IDouble1CalculatorHandler, IDouble2DoubleHandler
    {
        [HandlerParameter]
        public double Value
        {
            get;
            set;
        }

        private IList<double> MakeList(int count)
        {
            var res = new List<double>(count);
            for(int i = 0; i < count; i++)
            {
                res.Add(Value);
            }
            return res;
        }

        public IList<double> Execute(ISecurity source)
        {
            return MakeList(source.Bars.Count);
        }

        public double Execute(double source1)
        {
            return Value;
        }

        public IList<double> Execute(IList<double> source)
        {
            return MakeList(source.Count);
        }
    }

    [HandlerCategory("TradeMath")]
    public class Compress : IBar2BarHandler
    {
        [HandlerParameter(true, "5", Min = "5", Max = "60", Step = "5")]
        public int Interval
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source)
        {
            return source.CompressTo(new Interval(Interval, source.IntervalBase));
        }
    }

    [HandlerName("Compress to Seconds")]
    [HandlerCategory("TradeMath")]
    public class CompressToSeconds : IBar2BarHandler
    {
        [HandlerParameter(true, "5", Min = "5", Max = "60", Step = "5")]
        public int Interval
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source)
        {
            return source.CompressTo(new Interval(Interval, DataIntervals.SECONDS));
        }
    }

    [HandlerCategory("TradeMath")]
    public class Cutter : IBar2BarHandler
    {
        [HandlerParameter(true, "2", NotOptimized = true)]
        public int Decimals
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source)
        {
            var coef = Math.Pow(10, Decimals);
            var bars = new List<Bar>(source.Bars.Count);
            foreach (var bar in source.Bars)
            {
                var newBar = new Bar(bar.Color, bar.Date,
                                     Math.Ceiling(bar.Open*coef)/coef,
                                     Math.Ceiling(bar.High*coef)/coef,
                                     Math.Ceiling(bar.Low*coef)/coef,
                                     Math.Ceiling(bar.Close*coef)/coef,
                                     bar.Volume)
                             {Ask = Math.Ceiling(bar.Ask*coef)/coef, Bid = Math.Ceiling(bar.Bid*coef)/coef, AskQty = bar.AskQty, BidQty = bar.BidQty};
                bars.Add(newBar);
            }
            return source.CloneAndReplaceBars(bars);
        }
    }

    [HandlerName("Multiply")]
    [HandlerCategory("TradeMath")]
    public class SecMultiply : IBar2BarHandler
    {
        [HandlerParameter(true, "10")]
        public double Coef
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source)
        {
            var bars = new List<Bar>(source.Bars.Count);
            foreach (var bar in source.Bars)
            {
                var newBar = new Bar(bar.Color, bar.Date,
                                     Math.Ceiling(bar.Open*Coef),
                                     Math.Ceiling(bar.High*Coef),
                                     Math.Ceiling(bar.Low*Coef),
                                     Math.Ceiling(bar.Close*Coef),
                                     bar.Volume) { Ask = Math.Ceiling(bar.Ask * Coef), Bid = Math.Ceiling(bar.Bid * Coef), AskQty = bar.AskQty, BidQty = bar.BidQty };
                bars.Add(newBar);
            }
            return source.CloneAndReplaceBars(bars);
        }
    }

    [HandlerName("MultiplyWith")]
    [HandlerCategory("TradeMath")]
    public class SecMultiplyWith : IBarDouble2BarHandler
    {
        [HandlerParameter(true, "2", NotOptimized = true)]
        public int Decimals
        {
            get;
            set;
        }

        [HandlerParameter(true, "1")]
        public double Coef
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source, IList<double> src2)
        {
            var dcf = Math.Pow(10, Decimals);
            var bars = new List<Bar>(source.Bars.Count);
            for (int i = 0; i < source.Bars.Count; i++)
            {
                var bar = source.Bars[i];
                var s2 = src2.Count == 0 ? 0 : src2[Math.Min(i, src2.Count - 1)];
                var coef = Coef * s2 * dcf;
                var newBar = new Bar(bar.Color, bar.Date,
                                     Math.Ceiling(bar.Open*coef)/dcf,
                                     Math.Ceiling(bar.High*coef)/dcf,
                                     Math.Ceiling(bar.Low*coef)/dcf,
                                     Math.Ceiling(bar.Close*coef)/dcf,
                                     bar.Volume) { Ask = Math.Ceiling(bar.Ask * coef) / dcf, Bid = Math.Ceiling(bar.Bid * coef) / dcf, AskQty = bar.AskQty, BidQty = bar.BidQty };
                bars.Add(newBar);
            }
            return source.CloneAndReplaceBars(bars);
        }
    }

    [HandlerName("DivideWith")]
    [HandlerCategory("TradeMath")]
    public class SecDivideWith : IBarDouble2BarHandler
    {
        [HandlerParameter(true, "2", NotOptimized = true)]
        public int Decimals
        {
            get;
            set;
        }

        [HandlerParameter(true, "1")]
        public double Coef
        {
            get;
            set;
        }

        public ISecurity Execute(ISecurity source, IList<double> src2)
        {
            var dcf = Math.Pow(10, Decimals);
            var bars = new List<Bar>(source.Bars.Count);
            for (int i = 0; i < source.Bars.Count; i++)
            {
                var bar = source.Bars[i];
                var s2 = src2.Count == 0 ? 0 : src2[Math.Min(i, src2.Count - 1)];
                var coef = Coef * 1 / Math.Max(s2, 1e-10) * dcf;
                var newBar = new Bar(bar.Color, bar.Date,
                                     Math.Ceiling(bar.Open*coef)/dcf,
                                     Math.Ceiling(bar.High*coef)/dcf,
                                     Math.Ceiling(bar.Low*coef)/dcf,
                                     Math.Ceiling(bar.Close*coef)/dcf,
                                     bar.Volume) { Ask = Math.Ceiling(bar.Ask * coef) / dcf, Bid = Math.Ceiling(bar.Bid * coef) / dcf, AskQty = bar.AskQty, BidQty = bar.BidQty };
                bars.Add(newBar);
            }
            return source.CloneAndReplaceBars(bars);
        }
    }

    [HandlerName("Decompress")]
    [HandlerCategory("TradeMath")]
    public class Decompres : IDecompress, IBarDoubleAccumHandler
    {
        [HandlerParameter(true, "Default", NotOptimized = true)]
        public DecompressMethodWithDef Method { get; set; }

        public IList<double> Execute(ISecurity security, IList<double> source)
        {
            return security.Decompress(source, Method);
        }
    }

    [HandlerName("DecompressBool")]
    [HandlerCategory("TradeMath")]
    public class DecompresBool : IDecompress, IBarBoolAccumHandler
    {
        [HandlerParameter(true, "Default", NotOptimized = true)]
        public DecompressMethodWithDef Method { get; set; }

        public IList<bool> Execute(ISecurity security, IList<bool> source)
        {
            return security.Decompress(source, Method);
        }
    }

    [HandlerName("Absolute Commission")]
    [HandlerCategory("TradeMath")]
    public class AbsolutCommission : ICommissionHandler
    {
        [HandlerParameter(true, "0.0002", NotOptimized = true)]
        public double Commission
        {
            get;
            set;
        }

        public void Execute(ISecurity source)
        {
            source.Commission = ComissionDelegate;
        }

        private double ComissionDelegate(IPosition pos, bool isEntry)
        {
            return pos.Shares*Commission;
        }
    }

    [HandlerName("Relative Commission")]
    [HandlerCategory("TradeMath")]
    public class RelativeCommission : ICommissionHandler
    {
        [HandlerParameter(true, "0.05", NotOptimized = true)]
        public double CommissionPct
        {
            get;
            set;
        }

        [HandlerParameter(true, "10.0", NotOptimized = true)]
        public double MarginPct
        {
            get;
            set;
        }

        public void Execute(ISecurity source)
        {
            source.Commission = CommissionDelegate;
        }

        private double CommissionDelegate(IPosition pos, bool isEntry)
        {
            var price = isEntry ? pos.EntryPrice : pos.ExitPrice;
            var comm = price * CommissionPct/100.0;
            if(!isEntry && pos.IsShort)
            {
                var days = (pos.ExitBar.Date - pos.EntryBar.Date).TotalDays;
                comm += pos.EntryPrice*MarginPct/100*days/365; 
            }
            comm *= pos.Shares;
            return comm;
        }
    }
}
// ReSharper restore MemberCanBePrivate.Global
// ReSharper restore UnusedAutoPropertyAccessor.Global
// ReSharper restore UnusedMember.Global
